#Python 3.5-32
import ctypes
from ctypes import*

import os
import sys
import time
import subprocess

from threading import Thread

# Change current directory to PTS installation directory to simplify loading of DLLs and their dependencies 
os.chdir("C:\\Program Files (x86)\\Bluetooth SIG\\Bluetooth PTS\\bin")

# Prefix to identify final verdict string
VERDICT = 'VERDICT/'
 
# Verdict strings
RESULT_PASS = 'PASS'
RESULT_FAIL = 'FAIL'
RESULT_INCONC = 'INCONC'
RESULT_INCOMP = 'INCOMP' # Initial final verdict meaning that test has not completed yet
RESULT_NONE = 'NONE' # Error verdict usually indicating internal PTS error

SNIFFER_READY = 'SNIFFER/Save and clear complete'

# Output directory for sniffer files
DIRECTORY = b'C:\\Users\\mzhaksilikov\\Documents\\Profile Tuning Suite\\Test Directory'
DIRECTORY_STR = DIRECTORY.decode("utf-8")

# Global variables
test_result = RESULT_INCOMP
sniffer_ready = True
devices = []

# Prints devices found during device search
def print_found_devices(arg):
	step = 2
	duration = arg
	global devices
	while duration > 0:
		while len(devices) > 0:
			print(devices[0])
			del devices[0]
		time.sleep(step)
		duration -= step
		
# Looks for running Bluetooth Protocol Viewer process
# Returns True if finds one, False otherwise
def snifferRunning():
	prog=[line.split() for line in subprocess.check_output("tasklist").splitlines()]
	[prog.pop(e) for e in [0,1,2]] #useless 
	for task in prog:
		task_name = task[0].decode("utf-8")
		if task_name == "Fts.exe":
			print("Found FTS")
			time.sleep(1)
			return True
	return False

# Callback methods
# Defines which ImplicitSend will be used
# If returns True, local implementation will be used instead of the one
# specified by PTS
def UseAutoImplicitSend():
	return True
		
# Receives PTS dongle messages, specifically the ones
# telling that Bluetooth Protocol Viewer completed save/clear operation
def DongleMsg(msg_str):
	msg = (ctypes.c_char_p(msg_str).value).decode("utf-8")
	print(msg)
	time.sleep(1)
	global sniffer_ready
	if SNIFFER_READY in msg:
		print("FTS is ready")
		sniffer_ready = True
	return True

# Receives device search messages
# Each device may return multiple messages
# Each message will contain device address and may contain device name and COD
def DeviceSearchMsg(addr_str, name_str, cod_str):
	addr = (ctypes.c_char_p(addr_str).value).decode("utf-8")
	name = (ctypes.c_char_p(name_str).value).decode("utf-8")
	cod = (ctypes.c_char_p(cod_str).value).decode("utf-8")
	global devices
	devices.append("Device address = {0:s} name = {1:s} cod = {2:s}".format(addr, name, cod))
	return True

# PTS Log Types:
LOG_TYPE_GENERAL_TEXT			= 0
LOG_TYPE_FIRST					= 1 # first log type that may be toggled by user
LOG_TYPE_START_TEST_CASE		= 1
LOG_TYPE_TEST_CASE_ENDED		= 2
LOG_TYPE_START_DEFAULT			= 3
LOG_TYPE_DEFAULT_ENDED			= 4
LOG_TYPE_FINAL_VERDICT			= 5
LOG_TYPE_PRELIMINARY_VERDICT	= 6
LOG_TYPE_TIMEOUT				= 7	
LOG_TYPE_ASSIGNMENT				= 8
LOG_TYPE_START_TIMER			= 9	
LOG_TYPE_STOP_TIMER				= 10
LOG_TYPE_CANCEL_TIMER			= 11
LOG_TYPE_READ_TIMER				= 12
LOG_TYPE_ATTACH					= 13
LOG_TYPE_IMPLICIT_SEND			= 14
LOG_TYPE_GOTO					= 15
LOG_TYPE_TIMED_OUT_TIMER		= 16
LOG_TYPE_ERROR					= 17
LOG_TYPE_CREATE					= 18
LOG_TYPE_DONE					= 19
LOG_TYPE_ACTIVATE				= 20
LOG_TYPE_MESSAGE				= 21
LOG_TYPE_LINE_MATCHED			= 22
LOG_TYPE_LINE_NOT_MATCHED		= 23
LOG_TYPE_SEND_EVENT				= 24
LOG_TYPE_RECEIVE_EVENT			= 25
LOG_TYPE_OTHERWISE_EVENT		= 26
LOG_TYPE_RECEIVED_ON_PCO		= 27
LOG_TYPE_MATCH_FAILED			= 28
LOG_TYPE_COORDINATION_MESSAGE	= 29

# Receives log 	messages
def Log(log_time_str, log_descr_str, log_msg_str, log_type, project):
	log_time = (ctypes.c_char_p(log_time_str).value).decode("utf-8")
	log_descr = (ctypes.c_char_p(log_descr_str).value).decode("utf-8")
	log_msg = (ctypes.c_char_p(log_msg_str).value).decode("utf-8")
	print(log_time)
	log_msg = log_descr + log_msg
	print(log_msg)
	if ctypes.c_int(log_type).value == LOG_TYPE_FINAL_VERDICT:
		indx = log_msg.find(VERDICT)
		if indx == 0:
			global test_result
			print("Final verdict has been  received")
			if test_result == RESULT_INCOMP:
				if RESULT_INCONC in log_msg:
					test_result = RESULT_INCONC
				elif RESULT_FAIL in log_msg:
					test_result = RESULT_FAIL
				elif RESULT_PASS in log_msg:
					test_result = RESULT_PASS
				elif RESULT_NONE in log_msg:
					test_result = RESULT_NONE
	return True

# Implicit Send Styles:
#	MMI_Style_Ok_Cancel1 =     0x11041,	Simple prompt           | OK, Cancel buttons      | Default: OK
#	MMI_Style_Ok_Cancel2 =     0x11141,	Simple prompt           | Cancel button           | Default: Cancel
#	MMI_Style_Ok1 =            0x11040,	Simple prompt           | OK button               | Default: OK
#	MMI_Style_Yes_No1 =        0x11044,	Simple prompt           | Yes, No buttons         | Default: Yes
#	MMI_Style_Yes_No_Cancel1 = 0x11043,	Simple prompt           | Yes, No buttons         | Default: Yes
#	MMI_Style_Abort_Retry1 =   0x11042, Simple prompt           | Abort, Retry buttons    | Default: Abort
#	MMI_Style_Edit1 =          0x12040, Request for data input  | OK, Cancel buttons      | Default: OK
#	MMI_Style_Edit2 =          0x12140,	Select item from a list | OK, Cancel buttons      | Default: OK
#
# Handling
#	MMI_Style_Ok_Cancel1
#		OK = return "OK"
#		Cancel = return 0
#
#	MMI_Style_Ok_Cancel2
#		OK = return "OK" 
#		Cancel = return 0
#
#	MMI_Style_Ok1
#		OK = return "OK", this version should not return 0 
#
#	MMI_Style_Yes_No1
#		Yes = return "OK" 
#		No = return 0
#
#	MMI_Style_Yes_No_Cancel1
#		Yes = return "OK" 
#		No = return 0
#		Cancel = has been deprecated 
#
#	MMI_Style_Abort_Retry1
#		Abort = return 0 
#		Retry = return "OK"
#
#	MMI_Style_Edit1
#		OK = return expected string
#		Cancel = return 0
#
#	MMI_Style_Edit2
#		OK = return expected string
#		Cancel = return 0
#
# Receives ImplicitSend messages
# Description format is as following:
# {MMI_ID,Test Name,Layer Name}MMI Action\n\nDescription: MMI Description
def ImplicitSend(description, style):
	descr_str = (ctypes.c_char_p(description).value).decode("utf-8")
	#time.sleep(10)
	indx = descr_str.find('}')
	res = b''
	if indx != -1:
		implicitSendInfo = descr_str[1:(indx)]
		implicitSendDesc = descr_str[(indx + 1):]
		items = implicitSendInfo.split(',')
		implicitSendInfoID = items[0]
		implicitSendInfoTestCase = items[1]
		implicitSendInfoProject = items[2]
		print("OnImplicitSend() has been called with the following parameters:")
		print("")
		print("     project_name = {0:s}".format(implicitSendInfoProject))
		print("     id = {0:s}".format(implicitSendInfoID))
		print("     test_case = {0:s}".format(implicitSendInfoTestCase))
		print("     description = {0:s}".format(implicitSendDesc))
		print("     style = {0:#X}".format(ctypes.c_int(style).value))
		print("")
	style_val = ctypes.c_int(style).value
	res_str = ""
	if style_val == 0x11044:
		return None
	else:
		print("Other_style")
		res_str = "OK" 
		res = res_str.encode("utf-8")
		return res

# Main
# EtsManager dll path
ets_manager_library = "C:\\Program Files (x86)\\Bluetooth SIG\\Bluetooth PTS\\bin\\ETSManager.dll"
# Load EtsManager using absolute path
pts = cdll.LoadLibrary(ets_manager_library)
print("ETS Manager library {0:s} has been loaded".format(ets_manager_library))
print("")

# Define callback functions
USEAUTOIMPLSENDFUNC = CFUNCTYPE(c_bool)
use_auto_impl_send_func = USEAUTOIMPLSENDFUNC(UseAutoImplicitSend)

DONGLE_MSG_FUNC = CFUNCTYPE(c_bool, c_char_p)
dongle_msg_func = DONGLE_MSG_FUNC(DongleMsg)

DEVICE_SEARCH_MSG_FUNC = CFUNCTYPE(c_bool, c_char_p, c_char_p, c_char_p)
dev_search_msg_func = DEVICE_SEARCH_MSG_FUNC(DeviceSearchMsg)

LOGFUNC = CFUNCTYPE(c_bool, c_char_p, c_char_p, c_char_p, c_int, c_void_p)
log_func = LOGFUNC(Log)

ONIMPLSENDFUNC = CFUNCTYPE(c_char_p, c_char_p, c_int)
onimplsend_func = ONIMPLSENDFUNC(ImplicitSend)

# Register layer to test with callbacks
pts.RegisterProfileWithCallbacks.argtypes = [c_char_p, USEAUTOIMPLSENDFUNC, ONIMPLSENDFUNC, LOGFUNC, DEVICE_SEARCH_MSG_FUNC, DONGLE_MSG_FUNC]
res = pts.RegisterProfileWithCallbacks(b'A2DP', use_auto_impl_send_func, onimplsend_func, log_func, dev_search_msg_func, dongle_msg_func)

print("Profile has been registered with result {0:d}".format(res))
print("")

# GetDeviceInfo module is for discovering devices and PTS Dongle address
# Initialize GetDeviceInfo and register it with callbacks
# First parameter is PTS executable directory
pts.InitGetDevInfoWithCallbacks.argtypes = [c_char_p, DEVICE_SEARCH_MSG_FUNC, DONGLE_MSG_FUNC]
res = pts.InitGetDevInfoWithCallbacks(b'C:\\Program Files (x86)\\Bluetooth SIG\\Bluetooth PTS\\bin\\', dev_search_msg_func, dongle_msg_func)
print("GetDevInfo has been initialized with result {0:d}".format(res))
print("")

# Initialize connection to Protocol Viewer
pts.InitSniffer()

# Obtain the list of available radio devices
pts.GetDeviceList.restype = ctypes.c_char_p
res = pts.GetDeviceList()

sDeviceList = res.decode("utf-8")
if len(sDeviceList) < 1:
	print("{0:s}".format("Cannot find a device to connect"))
	exit()

aDeviceList = sDeviceList.split(';')
print("Device List: " + sDeviceList)

deviceToConnect = None

# look at each device
for sDevice in aDeviceList:
	# parse each device data
	# e.g.
	# COM format -> 'COM7'
	# USB format -> 'USB:Free:6&10EF065E&3&1' ('Free' or 'InUse')
	aInfo = sDevice.split(':')
	if (not ('USB:InUse' in aInfo[0])):
		deviceToConnect = aInfo[len(aInfo) - 1]
		break

if deviceToConnect is None:
	print("[ptsAutomation] none of the returned device was available for usage")
	exit()

print("Connecting to {0} ".format(deviceToConnect))        
pts.SetPTSDevice.argtypes = [ctypes.c_char_p]
pts.SetPTSDevice(deviceToConnect.encode("utf-8"))

# Initialize PTS dongle
res = pts.VerifyDongleEx()
print("PTS dongle has been initialized with result {0:d}".format(res))
print("")

# Find PTS dongle address
pts.GetDongleBDAddress.restype = ctypes.c_ulonglong
pts_address = pts.GetDongleBDAddress()
pts_address_str = "{0:012X}".format(pts_address)
print("PTS BD Address 0x{0:s}".format(pts_address_str))
print("")

# Get PTS dongle version
pts.GetDongleDeviceInformation()

time.sleep(2)

# Start function that prints found devices
# Function will be running for about 60 seconds
# thread = Thread(target = print_found_devices, args = (60,))
# thread.start()

# # Start device search
# pts.StartDeviceSearchEx.argtypes = [c_char_p, c_char_p, c_char_p]
# pts.StartDeviceSearchEx(b'000000', b'000000', b'GetDevInfo')

# # Wait till printing function completes 
# thread.join()

# # Stop device search	
# pts.StopDeviceSearchEx.argtypes = [c_char_p]
# pts.StopDeviceSearchEx(b'GetDevInfo');

# Initialize Bluetooth Protocol Viewer communication module
pts.SnifferInitializeEx();

# If Bluetooth Protocol Viewer is not running, start it
res = snifferRunning()
if res == False:
	print("Starting Protocol Viewer")
	args = ['C:\Program Files (x86)\Bluetooth SIG\Bluetooth Protocol Viewer\Executables\Core\FTS.exe', '/PTS Protocol Viewer=Generic', '/OEMTitle=Bluetooth Protocol Viewer', '/OEMKey=Virtual']
	subprocess.Popen(args)
	while res == False:
		time.sleep(10)
		res = snifferRunning()



# Register to recieve Bluetooth Protocol Viewer notofications		
pts.SnifferRegisterNotificationEx();

# Set ICS for selected layer
pts.SetParameterEx.restype = ctypes.c_bool
pts.SetParameterEx.argtypes = [c_char_p, c_char_p, c_char_p, c_char_p]
ics = {}
ics[b'TSPC_A2DP_1_1'] = b'FALSE'
ics[b'TSPC_A2DP_1_2'] = b'FALSE'
ics[b'TSPC_A2DP_2_1'] = b'FALSE'
ics[b'TSPC_A2DP_2_2'] = b'FALSE'
ics[b'TSPC_A2DP_2_3'] = b'FALSE'
ics[b'TSPC_A2DP_2_4'] = b'FALSE'
ics[b'TSPC_A2DP_2_5'] = b'TRUE'
ics[b'TSPC_A2DP_2_6'] = b'FALSE'
ics[b'TSPC_A2DP_2_7'] = b'FALSE'
ics[b'TSPC_A2DP_2_8'] = b'FALSE'
ics[b'TSPC_A2DP_2_9'] = b'FALSE'
ics[b'TSPC_A2DP_2_10'] = b'FALSE'
ics[b'TSPC_A2DP_2_10a'] = b'FALSE'
ics[b'TSPC_A2DP_2_11'] = b'FALSE'
ics[b'TSPC_A2DP_2_12'] = b'FALSE'
ics[b'TSPC_A2DP_2_13'] = b'FALSE'
ics[b'TSPC_A2DP_2_14'] = b'FALSE'
ics[b'TSPC_A2DP_2_15'] = b'FALSE'
ics[b'TSPC_A2DP_2_16'] = b'FALSE'
ics[b'TSPC_A2DP_2_17'] = b'FALSE'
ics[b'TSPC_A2DP_2a_1'] = b'FALSE'
ics[b'TSPC_A2DP_2a_2'] = b'FALSE'
ics[b'TSPC_A2DP_2a_3'] = b'FALSE'
ics[b'TSPC_A2DP_2b_1'] = b'FALSE'
ics[b'TSPC_A2DP_2b_2'] = b'FALSE'
ics[b'TSPC_A2DP_3_1'] = b'TRUE'
ics[b'TSPC_A2DP_3_1a'] = b'FALSE'
ics[b'TSPC_A2DP_3_2'] = b'FALSE'
ics[b'TSPC_A2DP_3_3'] = b'FALSE'
ics[b'TSPC_A2DP_3_4'] = b'FALSE'
ics[b'TSPC_A2DP_3_5'] = b'FALSE'
ics[b'TSPC_A2DP_3_6'] = b'FALSE'
ics[b'TSPC_A2DP_3_7'] = b'FALSE'
ics[b'TSPC_A2DP_3_8'] = b'FALSE'
ics[b'TSPC_A2DP_3a_1'] = b'TRUE'
ics[b'TSPC_A2DP_3a_2'] = b'FALSE'
ics[b'TSPC_A2DP_3a_3'] = b'FALSE'
ics[b'TSPC_A2DP_3a_4'] = b'FALSE'
ics[b'TSPC_A2DP_3a_5'] = b'FALSE'
ics[b'TSPC_A2DP_3a_6'] = b'FALSE'
ics[b'TSPC_A2DP_3a_7'] = b'FALSE'
ics[b'TSPC_A2DP_3a_8'] = b'FALSE'
ics[b'TSPC_A2DP_3a_9'] = b'FALSE'
ics[b'TSPC_A2DP_3a_10'] = b'FALSE'
ics[b'TSPC_A2DP_3a_11'] = b'FALSE'
ics[b'TSPC_A2DP_3a_12'] = b'TRUE'
ics[b'TSPC_A2DP_4_1'] = b'FALSE'
ics[b'TSPC_A2DP_4_2'] = b'TRUE'
ics[b'TSPC_A2DP_4_3'] = b'FALSE'
ics[b'TSPC_A2DP_4_4'] = b'TRUE'
ics[b'TSPC_A2DP_4_5'] = b'TRUE'
ics[b'TSPC_A2DP_4_6'] = b'FALSE'
ics[b'TSPC_A2DP_4_7'] = b'TRUE'
ics[b'TSPC_A2DP_4_8'] = b'FALSE'
ics[b'TSPC_A2DP_4_9'] = b'FALSE'
ics[b'TSPC_A2DP_4_10'] = b'TRUE'
ics[b'TSPC_A2DP_4_10a'] = b'FALSE'
ics[b'TSPC_A2DP_4_11'] = b'FALSE'
ics[b'TSPC_A2DP_4_12'] = b'FALSE'
ics[b'TSPC_A2DP_4_13'] = b'TRUE'
ics[b'TSPC_A2DP_4_14'] = b'TRUE'
ics[b'TSPC_A2DP_4_15'] = b'FALSE'
ics[b'TSPC_A2DP_5_1'] = b'TRUE'
ics[b'TSPC_A2DP_5_1a'] = b'FALSE'
ics[b'TSPC_A2DP_5_2'] = b'FALSE'
ics[b'TSPC_A2DP_5_3'] = b'FALSE'
ics[b'TSPC_A2DP_5_4'] = b'FALSE'
ics[b'TSPC_A2DP_5_5'] = b'FALSE'
ics[b'TSPC_A2DP_5a_1'] = b'TRUE'
ics[b'TSPC_A2DP_5a_2'] = b'TRUE'
ics[b'TSPC_A2DP_5a_3'] = b'TRUE'
ics[b'TSPC_A2DP_5a_4'] = b'TRUE'
ics[b'TSPC_A2DP_5a_5'] = b'TRUE'
ics[b'TSPC_A2DP_5a_6'] = b'TRUE'
ics[b'TSPC_A2DP_5a_7'] = b'TRUE'
ics[b'TSPC_A2DP_5a_8'] = b'TRUE'
ics[b'TSPC_A2DP_5a_9'] = b'TRUE'
ics[b'TSPC_A2DP_5a_10'] = b'TRUE'
ics[b'TSPC_A2DP_5a_11'] = b'TRUE'
ics[b'TSPC_A2DP_5a_12'] = b'TRUE'
ics[b'TSPC_A2DP_7a_1'] = b'FALSE'
ics[b'TSPC_A2DP_7a_2'] = b'FALSE'
ics[b'TSPC_A2DP_7a_3'] = b'FALSE'
ics[b'TSPC_A2DP_7b_1'] = b'FALSE'
ics[b'TSPC_A2DP_7b_2'] = b'FALSE'
ics[b'TSPC_A2DP_8_2'] = b'FALSE'
ics[b'TSPC_A2DP_8_3'] = b'FALSE'
ics[b'TSPC_A2DP_8_4'] = b'FALSE'
ics[b'TSPC_A2DP_9_1'] = b'FALSE'
ics[b'TSPC_A2DP_9_2'] = b'FALSE'
ics[b'TSPC_A2DP_9_3'] = b'FALSE'
ics[b'TSPC_A2DP_9_4'] = b'FALSE'
ics[b'TSPC_A2DP_10_1'] = b'FALSE'
ics[b'TSPC_A2DP_10_2'] = b'FALSE'
ics[b'TSPC_A2DP_10_3'] = b'FALSE'
ics[b'TSPC_A2DP_10_4'] = b'FALSE'
ics[b'TSPC_A2DP_10_5'] = b'FALSE'
ics[b'TSPC_A2DP_10_6'] = b'FALSE'
ics[b'TSPC_A2DP_11_1'] = b'FALSE'
ics[b'TSPC_A2DP_11_2'] = b'FALSE'
ics[b'TSPC_A2DP_11_3'] = b'FALSE'
ics[b'TSPC_A2DP_11_4'] = b'FALSE'
ics[b'TSPC_A2DP_11_5'] = b'FALSE'
ics[b'TSPC_A2DP_11_6'] = b'FALSE'
ics[b'TSPC_A2DP_12_2'] = b'FALSE'
ics[b'TSPC_A2DP_12_3'] = b'FALSE'
ics[b'TSPC_A2DP_12_4'] = b'FALSE'
ics[b'TSPC_A2DP_13_1'] = b'FALSE'
ics[b'TSPC_A2DP_13_2'] = b'FALSE'
ics[b'TSPC_A2DP_13_3'] = b'FALSE'
ics[b'TSPC_A2DP_13_4'] = b'FALSE'
ics[b'TSPC_A2DP_14_1'] = b'FALSE'
ics[b'TSPC_A2DP_14_2'] = b'FALSE'
ics[b'TSPC_A2DP_14_3'] = b'FALSE'
ics[b'TSPC_A2DP_14_4'] = b'FALSE'
ics[b'TSPC_A2DP_14_5'] = b'FALSE'
ics[b'TSPC_A2DP_15_1'] = b'FALSE'
ics[b'TSPC_A2DP_15_2'] = b'FALSE'
ics[b'TSPC_A2DP_15_3'] = b'FALSE'
ics[b'TSPC_A2DP_15_4'] = b'FALSE'
ics[b'TSPC_A2DP_15_5'] = b'FALSE'
ics[b'TSPC_A2DP_15_6'] = b'FALSE'
ics[b'TSPC_ALL'] = b'FALSE'

for ics_name in ics:
	res = pts.SetParameterEx(ics_name, b'BOOLEAN', ics[ics_name], b'A2DP')
	if res == True:
		print("ICS {0:s} set successfully".format(str(ics_name)))
	else:
		print("Setting ICS {0:s} value failed".format(str(ics_name)))
print("")

# Set IXIT for selected layer
ixit = {}
ixit[b'TSPX_security_enabled'] = (b'BOOLEAN',  b'FALSE')
ixit[b'TSPX_bd_addr_iut'] = (b'OCTETSTRING',  b'001BDC08C236')
ixit[b'TSPX_SRC_class_of_device'] = (b'OCTETSTRING',  b'080418')
ixit[b'TSPX_SNK_class_of_device'] = (b'OCTETSTRING',  b'04041C')
ixit[b'TSPX_pin_code'] = (b'IA5STRING',  b'0000')
ixit[b'TSPX_delete_link_key'] = (b'BOOLEAN',  b'FALSE')
ixit[b'TSPX_time_guard'] = (b'INTEGER',  b'300000')
ixit[b'TSPX_use_implicit_send'] = (b'BOOLEAN',  b'TRUE')
ixit[b'TSPX_media_directory'] = (b'IA5STRING',  b'C:\Program Files (x86)\Bluetooth SIG\Bluetooth PTS\bin\audio')
ixit[b'TSPX_auth_password'] = (b'IA5STRING',  b'0000')
ixit[b'TSPX_auth_user_id'] = (b'IA5STRING',  b'PTS')
ixit[b'TSPX_rfcomm_channel'] = (b'INTEGER',  b'8')
ixit[b'TSPX_l2cap_psm'] = (b'OCTETSTRING',  b'1011')
ixit[b'TSPX_no_confirmations'] = (b'BOOLEAN',  b'FALSE')
ixit[b'TSPX_cover_art_uuid'] = (b'OCTETSTRING',  b'3EEE')

for ixit_name in ixit:
	res = pts.SetParameterEx(ixit_name, (ixit[ixit_name])[0], (ixit[ixit_name])[1], b'A2DP')
	if res == True:
		print("IXIT {0:s} set successfully".format(str(ixit_name)))
	else:
		print("Setting IXIT {0:s} value failed".format(str(ixit_name)))
print("")

# Prepare directory to store Bluetooth Protocol Viewer output
if not os.path.exists(DIRECTORY_STR):
    os.makedirs(DIRECTORY_STR)

# Initialize selected layer DLL
pts_address_b = pts_address_str.encode("utf-8")
pts.InitEtsEx.restype = ctypes.c_bool
pts.InitEtsEx.argtypes = [c_char_p, c_char_p, c_char_p, c_char_p]
res = pts.InitEtsEx(b'A2DP', DIRECTORY, b'C:\\Program Files (x86)\\Bluetooth SIG\\Bluetooth PTS\\bin\\implicit_send3.dll', pts_address_b)
print("ETS has been initialized with result {0:s}".format(str(res)))

# Initialize Host Stack DLL
pts.InitStackEx.restype = ctypes.c_bool
pts.InitStackEx.argtypes = [c_char_p]
res = pts.InitStackEx(b'A2DP')
print("Stack has been initialized with result {0:s}".format(str(res)))

# Select to receive Log messages after test is done
pts.SetPostLoggingEx.argtypes = [c_bool, c_char_p]
pts.SetPostLoggingEx(True, b'A2DP')

# Clear Bluetooth Protocol Viewer
sniffer_ready = False
pts.SnifferClearEx()
# while sniffer_ready == False:
	# time.sleep(1)

# Start selected test
pts.StartTestCaseEx.restype = ctypes.c_bool
pts.StartTestCaseEx.argtypes = [c_char_p, c_char_p, c_bool]
res = pts.StartTestCaseEx(b'A2DP/SNK/AS/BV-01-I', b'A2DP', True)
print("Test has been started with result {0:s}".format(str(res)))

# Wait till verdict is received
while test_result == RESULT_INCOMP:
	time.sleep(1)
	
# Clean up after test is done
pts.TestCaseFinishedEx.restype = ctypes.c_bool
pts.TestCaseFinishedEx.argtypes = [c_char_p, c_char_p]
res = pts.TestCaseFinishedEx(b'A2DP/SNK/AS/BV-01-I', b'A2DP')

# Save collected Bluetooth Protocol Viwer data
pts.SnifferIsRunningEx.restype = ctypes.c_bool
res = ctypes.c_bool(pts.SnifferIsRunningEx()).value
if res == True:
	pts.SnifferCanSaveEx.restype = ctypes.c_bool
	canSave = ctypes.c_bool(pts.SnifferCanSaveEx()).value
	pts.SnifferCanSaveAndClearEx.restype = ctypes.c_bool
	canSaveClear = ctypes.c_bool(pts.SnifferCanSaveAndClearEx()).value
	file_name = b'\\Log.cfa'
	path = DIRECTORY + file_name[0:8]
	if canSave == True:
		pts.SnifferSaveEx.argtypes = [c_char_p]
		pts.SnifferSaveEx(path)
	else:
		pts.SnifferSaveAndClearEx.argtypes = [c_char_p]
		pts.SnifferSaveAndClearEx(path)
	while sniffer_ready == False:
		time.sleep(1)

# Call clean up functions after overall testing is done
pts.ExitStackEx('A2DP');
pts.UnregisterProfileEx.argtypes = [c_char_p]
pts.UnregisterProfileEx(b'A2DP')
pts.UnRegisterGetDevInfoEx()
pts.SnifferTerminateEx()		

# Done


