/***
 *** HelperDialog.h
 ***
 *** This is the header file for the PTS helper dialog.  This file should not be included as part of the Implicit
 *** Send Project distributed through PTS.
 ***
 *** ----------
 ***
 *** This module provides the dialog box that is used to interact with the test operator.
 ***
 *** ----------
 ***
 *** Copyright 2012, Bluetooth SIG, Inc.
 ***
 *** ----------
 ***
 *** Bradford Ayres
 *** Bluetooth SIG, Inc.
 ***/

#if	(!defined(HELPERDIALOG_H_INCLUDED))
#define	HELPERDIALOG_H_INCLUDED

/****************************************************************************************************************************
 **
 ** Includes (needed by the class)
 **/
#include	<string>
#include	<list>
#include    <map>
#include	"mycstatic.h"
#import		<msxml6.dll> exclude("IStream", "ISequentialStream", "_LARGE_INTEGER", "_ULARGE_INTEGER", "tagSTATSTG", "_FILETIME")

/****************************************************************************************************************************
 **
 ** ImplicitSendDialog Class
 **/

std::string RunHelperDialog( const std::string &xmlString, const std::string &strBdAddr );

class CHelperDialog : public CDialog
{
/*
 * Construction/Destruction
 */
public:
	CHelperDialog( const std::string &xmlString = "", HANDLE* hEvents = NULL );
	~CHelperDialog(void);

/*
 * Interface
 */
public:
	
/*
 * Class internal functions
 */
private:
	/*
	 * Window message handlers
	 */
	void		DoDataExchange(CDataExchange* pDx);
	void		OnCommandListDoubleClick(void);
	void		OnCommandSelectionChanged(void);
	BOOL		OnInitDialog(void);
	void		OnClose(void);
	DECLARE_MESSAGE_MAP();

	// Support Methods
	void SetActiveFilter( const std::string &filterName );
	std::string InitializeFromXMLString( void );
	void InitializeHelpers( std::string &helperList );
	void InitializeCommand( const std::string &helperName, MSXML2::IXMLDOMNodePtr helperNode );

	void ClearCommandListBox( void );
	void ExecuteCommand( const std::string &commandName );
	void DisplayCommandDescription( void );

private :

	struct Command {
	public :
		Command( const std::string &helperName = "", const std::string &commandName = "", 
			const std::string &role = "", const std::string &description = "" ) :
			m_HelperName(helperName), m_Role(role), m_Description( description )
		{
			if( m_Role.empty() )
			{
				m_CommandName = m_HelperName + ": " + commandName;
			}
			else
			{
				m_CommandName = m_HelperName + "(" + m_Role + "): " + commandName;
			}
		}
		
		std::string m_HelperName;
		std::string m_CommandName;
		std::string m_Role;
		std::string m_Description;
	};

	class Filter {
	public :

		Filter( void ){}

		void SetActiveFilter( CListBox &listBox );
		void AddCommand( Command* pComand );

		std::map<std::string, Command*> m_Commands;
	};

/*
 * Class data
 */
private:
	/*
	 * Controls
	 */
	//CStatic		m_HelperDescription;   // Text area to display helper description
	CStatic     m_CommandDescription;  // Text area to dispaly command description
	CListBox    m_CommandListBox;      // List box used to hold helper commands
	CComboBox	m_FilterComboBox;      // Combo Box used to select the current filter

	// Command/Filter members
	std::list<CHelperDialog::Command*> m_Commands;
	std::map<std::string, Filter> m_Filters;

	HANDLE m_hEvents[1];

	// Initialization Information
	std::string m_InitializationXmlString; // XML Text used to initialize the MMI
public:
	afx_msg void OnCbnSelchangeHelperFilter();
};
#endif
