/***
 *** ImplicitSendDialog.cpp
 ***
 *** This is the source code for the standard implicit_send3.dll that is provided with the Profile Tuning Suite (PTS). It
 *** is provided to PTS users as a starting point for developing their own Implict Send DLL.
 ***
 *** Please see the "Automating - Using Implicit Send" document for more information. This document can be found in the
 *** same folder as this source code, or by selecting "Reference Documents" -> "Automating" from the PTS Start page.
 ***
 *** ----------
 ***
 *** This module provides the dialog box that is used to interact with the test operator.
 ***
 *** ----------
 ***
 *** Copyright 2012, Bluetooth SIG, Inc.
 ***
 *** ----------
 ***
 *** Tom Allebrandi (tallebrandi@bluetooth.com)
 *** Bluetooth SIG, Inc.
 ***/


/****************************************************************************************************************************
 **
 ** Include Files
 **/
#define	WINVER	0x0501	// Windows XP

#include	<afxwin.h>
#include	<string>
#include	<vector>

#include	"implicitsendapi.h"
#include	"implicitsenddialog.h"
#include	"mycstatic.h"
#include	"resource.h"
#include	<shlObj.h>

#define MAX_WIDTH 1024
#define MAX_HEIGHT 768


/****************************************************************************************************************************
 **
 ** Data
 **/
/*
 * MFC requires an instance of a CWinApp object to hold state information in.
 */
CWinApp thisDll;
std::string	m_strBdAddr;	// Bluetooth Address apend to the caption

/****************************************************************************************************************************
 **
 ** ImplicitSendDialog MessageMap
 **/

BEGIN_MESSAGE_MAP(ImplicitSendDialog,CDialog)
ON_BN_CLICKED(IDC_BUTTON1,OnButton1)
ON_BN_CLICKED(IDC_BUTTON2,OnButton2)
ON_LBN_DBLCLK(IDC_PICK_LIST,OnButton1)
ON_MESSAGE(WM_IMPLICITSENDDIALOG, OnMessageReceived) 
ON_WM_CLOSE()
END_MESSAGE_MAP()


/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::ImplicitSendDialog()
 **
 ** Class constructor
 **/

ImplicitSendDialog::ImplicitSendDialog() :
	CDialog(IDD_IMPLICITSEND)
{
/*
 * Initialize member data
 */
m_mmiStyle = 0;
m_strMmiText.clear();
m_strResponse.clear();
m_strBdAddr.clear();
}


/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::~ImplicitSendDialog()
 **
 ** Class destructor
 **/

ImplicitSendDialog::~ImplicitSendDialog()
{
/*
 * We don't have anything that needs to be cleaned up
 */
}


/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::GetData()
 **
 ** Return the response data stored by styles MMI_Style_Edit1 or MMI_Style_Edit2
 **/

std::string ImplicitSendDialog::GetData()
{
/*
 * Just return the member data
 */
return(m_strResponse);
}


/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::SetData()
 **
 ** Copy the data that was supplied to ImplicitSendStyle() into the class
 **/

void ImplicitSendDialog::SetData(
	const std::string&	strMmiText,
	UINT				mmiStyle,
	const std::string&	strBdAddr)
{
/*
 * Just set the member data
 */
m_strMmiText = strMmiText;
m_mmiStyle = mmiStyle;
m_strBdAddr = strBdAddr;
}


/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::DoDataExchange()
 **
 ** Bind the resource IDs from the dialog template to variables for the controls
 **/

void ImplicitSendDialog::DoDataExchange(
	CDataExchange	*pDx)
{
/*
 * Do the default stuff first
 */
CDialog::DoDataExchange(pDx);

/*
 * Connect the controls to variables
 */
DDX_Control(pDx,IDC_BUTTON1,m_cButton1);
DDX_Control(pDx,IDC_BUTTON2,m_cButton2);
DDX_Control(pDx,IDC_MESSAGE_AREA,m_cMessageArea);
DDX_Control(pDx,IDOK,m_cOK);
DDX_Control(pDx,IDC_PICK_LIST,m_cPickList);
DDX_Control(pDx,IDC_TAG_AREA,m_cTagArea);
DDX_Control(pDx,IDC_USER_INPUT,m_cUserInput);
}

/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::OnMessageReceived()
 **
 ** Message handler for EndDialog
 **/
LRESULT ImplicitSendDialog::OnMessageReceived(WPARAM wParam, LPARAM lParam)
{ 
	// Without the below line, the compiler will complain.
	UNREFERENCED_PARAMETER( lParam );

	EndDialog(static_cast<int>(wParam));
    return 0; 
}

/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::OnButton1()
 **
 ** Message handler for Button 1
 **/

void ImplicitSendDialog::OnButton1()
{
	CString				csWindowText;
	int					index;
	bool				terminateDialog = false;

/*
 * Button 1 always means that there is something to be returned. It may be user input, selection from a list, or just an
 * affirmative request to end the dialog. Set the result value based on the selected style
 */
switch (m_mmiStyle)
 {
 case MMI_Style_Edit1:
	{
	/*
	 * The user has entered some data into the edit control. That's what we need to return
	 */
	m_cUserInput.GetWindowText(csWindowText);

	/*
	 * Copy the data to the response member
	 */
	m_strResponse = std::string(csWindowText.GetBuffer());
	csWindowText.ReleaseBuffer();
	terminateDialog = true;
	break;
	}

 case MMI_Style_Edit2:
	{
	/*
	 * The user has (may have) picked an item from the option list. Retreive it
	 */
	if ((index=m_cPickList.GetCurSel()) != LB_ERR)
		{
		/*
		 * The user has picked something. Get the data from the control
		 */
		m_cPickList.GetText(index,csWindowText);

		/*
		 * Copy the data to the response member
		 */
		m_strResponse = std::string(csWindowText.GetBuffer());
		csWindowText.ReleaseBuffer();
		terminateDialog = true;
		}
	 else
		{
		/*
		 * The user has hit Button 1 without choosing anything. Ignore the button press
		 */
		m_cPickList.SetFocus();
		terminateDialog = false;
		}

	break;
	}

 default:
	{
	/*
	 * All of the other styles are simply an affirmative request to the end the dialog
	 */
	m_strResponse = "OK";	// Just in case
	terminateDialog = true;
	}
 }

/*
 * Are we terminating?
 */
if (terminateDialog)
	{
	/*
	 * Yes
	 */

	/*
	 * EndDialog must be called within ui-thread, otherwise it would not be synchronized with RunModalLoop
	 * EndDialog(AffirmativeResponse);
	 */
	PostMessage(WM_IMPLICITSENDDIALOG, AffirmativeResponse);
	}
}


/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::OnButton2()
 **
 ** Message handler for Button 2
 **/

void ImplicitSendDialog::OnButton2()
{
/*
 * Button 2 is almost always some form of cancellation or termination request so there is no information to be returned, we
 * can just terminate the dialog. The exception to the rule is MMI_Style_OK where Button 2 is an affirmative response
 */
if (m_mmiStyle == MMI_Style_Ok1)
	{
	m_strResponse = "OK";	// Just in case

	/*
	 * EndDialog must be called within ui-thread, otherwise it would not be synchronized with RunModalLoop
	 * EndDialog(AffirmativeResponse);
	 */
	PostMessage(WM_IMPLICITSENDDIALOG, AffirmativeResponse);
	}
 else
	{
	/*
	 * EndDialog must be called within ui-thread, otherwise it would not be synchronized with RunModalLoop
	 * EndDialog(NegativeResponse);
	 */
	PostMessage(WM_IMPLICITSENDDIALOG, NegativeResponse);
	}
}


/****************************************************************************************************************************
 **
 ** MainDialog::OnCancel()
 **
 ** From section 5.1 of "Automating - Using Implicit Send":
 **
 **     "At various points during the execution of a test, the test case implementation can detect that an action requested
 **      via Implicit Send has occurred. When this happens, the test case may attempt to complete the ImplicitSendStyle()
 **      or ImplicitSendPinCode() operation that is in progress.
 **
 **      The mechanism used to do this is to send simulated button presses to a dialog whose title is User Action
 **      Required. ..."
 **
 ** What is actually sent from the test case is a WM_LBUTTONDOWN to the button whose ID is either IDOK or IDCANCEL depending
 ** on a parameter that is passed to the function to close the message box. The WM_LBUTTONDOWN is followed by a WM_LBUTTONUP
 ** to the same button.
 **
 ** We have hidden buttons on the dialog with IDs of IDOK and IDCANCEL to support this functionality. This is the handler for
 ** the IDCANCEL button
 **/

void ImplicitSendDialog::OnCancel()
{
/*
 * Terminate the dialog
 */
m_strResponse.clear();
/*
 * EndDialog must be called within ui-thread, otherwise it would not be synchronized with RunModalLoop
 * EndDialog(DialogCancelTerminationRequestedByTestCase);
 */
PostMessage(WM_IMPLICITSENDDIALOG, DialogCancelTerminationRequestedByTestCase);
}


/****************************************************************************************************************************
 **
 ** MainDialog::OnInitDialog()
 **
 ** Dialog initialization routine
 **
 ** For consistency of presentation, we are using one dialog template for all of the MMI styles. Depending on the style
 ** we set the labels for Button 1 and Button 2, enable or disable some of the controls, move some of the controls around,
 ** change the size of the dialog, etc.
 **/

BOOL ImplicitSendDialog::OnInitDialog()
{
	bool								disableSysModal = false;
	bool								focusSet = false;
	int									heightAdjustment;
	std::vector<std::string>::size_type	index;
	std::vector<std::string>			optionList;
	FILE*								pLogFile = NULL;
	bool								showBorder = true;
	bool								showTag = false;
	std::string							strMessage;
	std::string							strTag;
	RECT								windowRect;

/*
 * Do the default stuff first
 */
CDialog::OnInitDialog();

/*
 * Set the icon for the dialog
 */
SetIcon(AfxGetApp()->LoadIcon(IDI_BLUETOOTH_LOGO),TRUE);

/*
 * Get the options from PTS.ini
 *
 * disableSysModal: The dialog template has the System Modal (DS_SYSMODAL) flag set. Sometimes that setting is inconvenient.
 * showTag: Each MMI has a identification tag. This flag will show it on the dialog
 */
getOptionFlags(disableSysModal,showTag,showBorder);

/*
 * The MMI text string that was send into ImplicitSendStyle() may have three sections in it:
 *
 * 1) The unique ID for the MMI
 * 2) The message to be displayed
 * 3) The list of options for MMI_Style_Edit2
 *
 * Extract the various parts
 */
processMmiText(m_strMmiText,m_mmiStyle,strMessage,strTag,optionList);

/*
 * Add the border to message area if requested in the options
 */
if (showBorder)
	{
	m_cMessageArea.ModifyStyle(0,WS_BORDER);
	adjustWindow(m_cMessageArea,0,0,0,1);	// If we don't adjust the window size, the border won't appear
	adjustWindow(m_cMessageArea,0,0,0,-1);
	}

/*
 * Configure the dialog based on the style. This method is somewhat brute force, but I think it will make it easier to
 * maintain
 */
switch (m_mmiStyle)
 {
 case MMI_Style_Ok_Cancel1:		// Simple prompt | OK, Cancel buttons | Default: OK
	{
	/*
	 * Button 1
	 */
	m_cButton1.EnableWindow(TRUE);
	m_cButton1.ShowWindow(SW_SHOW);
	m_cButton1.SetWindowText("OK");
	SetDefID(m_cButton1.GetDlgCtrlID());

	/*
	 * Button 2
	 */
	m_cButton2.EnableWindow(TRUE);
	m_cButton2.ShowWindow(SW_SHOW);
	m_cButton2.SetWindowText("Cancel");

	/*
	 * Message area
	 */
	m_cMessageArea.EnableWindow(TRUE);
	m_cMessageArea.ShowWindow(SW_SHOW);

	/*
	 * Hidden OK button
	 */
	m_cOK.EnableWindow(TRUE);
	m_cOK.ShowWindow(SW_HIDE);

	/*
	 * List of options -- Not used in this style
	 */
	m_cPickList.EnableWindow(FALSE);
	m_cPickList.ShowWindow(SW_HIDE);

	/*
	 * Tag area
	 */
	if (showTag)
		{
		m_cTagArea.EnableWindow(TRUE);
		m_cTagArea.ShowWindow(SW_SHOW);
		m_cTagArea.SetWindowText(strTag.c_str());
		}
	 else
		{
		m_cTagArea.EnableWindow(FALSE);
		m_cTagArea.ShowWindow(SW_HIDE);
		}

	/*
	 * User input box -- Not used in this style
	 */
	m_cUserInput.EnableWindow(FALSE);
	m_cUserInput.ShowWindow(SW_HIDE);

	break;
	}

 case MMI_Style_Ok_Cancel2:		// Simple prompt | Cancel button | Default: Cancel
	{
	/*
	 * Button 1 -- Not used in this style
	 */
	m_cButton1.EnableWindow(FALSE);
	m_cButton1.ShowWindow(SW_HIDE);

	/*
	 * Button 2
	 */
	m_cButton2.EnableWindow(TRUE);
	m_cButton2.ShowWindow(SW_SHOW);
	m_cButton2.SetWindowText("Cancel");
	SetDefID(m_cButton2.GetDlgCtrlID());

	/*
	 * Message area
	 */
	m_cMessageArea.EnableWindow(TRUE);
	m_cMessageArea.ShowWindow(SW_SHOW);

	/*
	 * Hidden OK button
	 */
	m_cOK.EnableWindow(TRUE);
	m_cOK.ShowWindow(SW_HIDE);

	/*
	 * List of options -- Not used in this style
	 */
	m_cPickList.EnableWindow(FALSE);
	m_cPickList.ShowWindow(SW_HIDE);

	/*
	 * Tag area
	 */
	if (showTag)
		{
		m_cTagArea.EnableWindow(TRUE);
		m_cTagArea.ShowWindow(SW_SHOW);
		m_cTagArea.SetWindowText(strTag.c_str());
		}
	 else
		{
		m_cTagArea.EnableWindow(FALSE);
		m_cTagArea.ShowWindow(SW_HIDE);
		}

	/*
	 * User input box -- Not used in this style
	 */
	m_cUserInput.EnableWindow(FALSE);
	m_cUserInput.ShowWindow(SW_HIDE);

	break;
	}

 case MMI_Style_Ok1:			// Simple prompt | OK button | Default: OK
	{
	/*
	 * Button 1 -- Not used in this style
	 */
	m_cButton1.EnableWindow(FALSE);
	m_cButton1.ShowWindow(SW_HIDE);

	/*
	 * Button 2
	 */
	m_cButton2.EnableWindow(TRUE);
	m_cButton2.ShowWindow(SW_SHOW);
	m_cButton2.SetWindowText("OK");
	SetDefID(m_cButton1.GetDlgCtrlID());

	/*
	 * Message area
	 */
	m_cMessageArea.EnableWindow(TRUE);
	m_cMessageArea.ShowWindow(SW_SHOW);

	/*
	 * Hidden OK button
	 */
	m_cOK.EnableWindow(TRUE);
	m_cOK.ShowWindow(SW_HIDE);

	/*
	 * List of options -- Not used in this style
	 */
	m_cPickList.EnableWindow(FALSE);
	m_cPickList.ShowWindow(SW_HIDE);

	/*
	 * Tag area
	 */
	if (showTag)
		{
		m_cTagArea.EnableWindow(TRUE);
		m_cTagArea.ShowWindow(SW_SHOW);
		m_cTagArea.SetWindowText(strTag.c_str());
		}
	 else
		{
		m_cTagArea.EnableWindow(FALSE);
		m_cTagArea.ShowWindow(SW_HIDE);
		}

	/*
	 * User input box -- Not used in this style
	 */
	m_cUserInput.EnableWindow(FALSE);
	m_cUserInput.ShowWindow(SW_HIDE);
	break;
	}

 case MMI_Style_Yes_No1:		// Simple prompt | Yes, No buttons         | Default: Yes
 case MMI_Style_Yes_No_Cancel1:	// Simple prompt | Yes, No, Cancel buttons | Default: Yes
								// The Cancel button did the same thing as the No button, so we dropped it
	{
	/*
	 * Button 1
	 */
	m_cButton1.EnableWindow(TRUE);
	m_cButton1.ShowWindow(SW_SHOW);
	m_cButton1.SetWindowText("Yes");
	SetDefID(m_cButton1.GetDlgCtrlID());

	/*
	 * Button 2
	 */
	m_cButton2.EnableWindow(TRUE);
	m_cButton2.ShowWindow(SW_SHOW);
	m_cButton2.SetWindowText("No");

	/*
	 * Message area
	 */
	m_cMessageArea.EnableWindow(TRUE);
	m_cMessageArea.ShowWindow(SW_SHOW);

	/*
	 * Hidden OK button
	 */
	m_cOK.EnableWindow(TRUE);
	m_cOK.ShowWindow(SW_HIDE);

	/*
	 * List of options -- Not used in this style
	 */
	m_cPickList.EnableWindow(FALSE);
	m_cPickList.ShowWindow(SW_HIDE);

	/*
	 * Tag area
	 */
	if (showTag)
		{
		m_cTagArea.EnableWindow(TRUE);
		m_cTagArea.ShowWindow(SW_SHOW);
		m_cTagArea.SetWindowText(strTag.c_str());
		}
	 else
		{
		m_cTagArea.EnableWindow(FALSE);
		m_cTagArea.ShowWindow(SW_HIDE);
		}

	/*
	 * User input box -- Not used in this style
	 */
	m_cUserInput.EnableWindow(FALSE);
	m_cUserInput.ShowWindow(SW_HIDE);

	break;
	}

 case MMI_Style_Abort_Retry1:	// Simple prompt | Abort, Retry buttons | Default: Abort
								// This style used to also have an Ingore button, but it did the same thing that the
								// Retry button does, so we dropped it
	{
	/*
	 * Button 1
	 */
	m_cButton1.EnableWindow(TRUE);
	m_cButton1.ShowWindow(SW_SHOW);
	m_cButton1.SetWindowText("Retry");

	/*
	 * Button 2
	 */
	m_cButton2.EnableWindow(TRUE);
	m_cButton2.ShowWindow(SW_SHOW);
	m_cButton2.SetWindowText("Abort");
	SetDefID(m_cButton1.GetDlgCtrlID());

	/*
	 * Message area
	 */
	m_cMessageArea.EnableWindow(TRUE);
	m_cMessageArea.ShowWindow(SW_SHOW);

	/*
	 * Hidden OK button
	 */
	m_cOK.EnableWindow(TRUE);
	m_cOK.ShowWindow(SW_HIDE);

	/*
	 * List of options -- Not used in this style
	 */
	m_cPickList.EnableWindow(FALSE);
	m_cPickList.ShowWindow(SW_HIDE);

	/*
	 * Tag area
	 */
	if (showTag)
		{
		m_cTagArea.EnableWindow(TRUE);
		m_cTagArea.ShowWindow(SW_SHOW);
		m_cTagArea.SetWindowText(strTag.c_str());
		}
	 else
		{
		m_cTagArea.EnableWindow(FALSE);
		m_cTagArea.ShowWindow(SW_HIDE);
		}

	/*
	 * User input box -- Not used in this style
	 */
	m_cUserInput.EnableWindow(FALSE);
	m_cUserInput.ShowWindow(SW_HIDE);

	break;
	}

 case MMI_Style_Edit1:			// Request for data input | OK, Cancel buttons | Default: OK
	{
	/*
	 * Button 1
	 */
	m_cButton1.EnableWindow(TRUE);
	m_cButton1.ShowWindow(SW_SHOW);
	m_cButton1.SetWindowText("OK");
	SetDefID(m_cButton1.GetDlgCtrlID());

	/*
	 * Button 2
	 */
	m_cButton2.EnableWindow(TRUE);
	m_cButton2.ShowWindow(SW_SHOW);
	m_cButton2.SetWindowText("Cancel");

	/*
	 * Message area
	 */
	m_cMessageArea.EnableWindow(TRUE);
	m_cMessageArea.ShowWindow(SW_SHOW);

	/*
	 * Hidden OK button
	 */
	m_cOK.EnableWindow(TRUE);
	m_cOK.ShowWindow(SW_HIDE);

	/*
	 * List of options -- Not used in this style
	 */
	m_cPickList.EnableWindow(FALSE);
	m_cPickList.ShowWindow(SW_HIDE);

	/*
	 * Tag area
	 */
	if (showTag)
		{
		m_cTagArea.EnableWindow(TRUE);
		m_cTagArea.ShowWindow(SW_SHOW);
		m_cTagArea.SetWindowText(strTag.c_str());
		}
	 else
		{
		m_cTagArea.EnableWindow(FALSE);
		m_cTagArea.ShowWindow(SW_HIDE);
		}

	/*
	 * User input box
	 */
	m_cUserInput.EnableWindow(TRUE);
	m_cUserInput.ShowWindow(SW_SHOW);
	m_cUserInput.SetFocus();
	focusSet = true;

	/*
	 * The message area overlaps the user input box. Reduce the height of the message area by 1.5 times the height of the
	 * user area
	 */
	adjustWindow(m_cMessageArea,0,0,0,(int) (-1.5 * getWindowHeight(m_cUserInput)));
	break;
	}

 case MMI_Style_Edit2:			// Select item from a list | OK, Cancel buttons | Default: OK
	{
	/*
	 * Button 1
	 */
	m_cButton1.EnableWindow(TRUE);
	m_cButton1.ShowWindow(SW_SHOW);
	m_cButton1.SetWindowText("OK");
	SetDefID(m_cButton1.GetDlgCtrlID());

	/*
	 * Button 2
	 */
	m_cButton2.EnableWindow(TRUE);
	m_cButton2.ShowWindow(SW_SHOW);
	m_cButton2.SetWindowText("Cancel");

	/*
	 * Message area
	 */
	m_cMessageArea.EnableWindow(TRUE);
	m_cMessageArea.ShowWindow(SW_SHOW);

	/*
	 * Hidden OK button
	 */
	m_cOK.EnableWindow(TRUE);
	m_cOK.ShowWindow(SW_HIDE);

	/*
	 * List of options
	 */
	m_cPickList.EnableWindow(TRUE);
	m_cPickList.ShowWindow(SW_SHOW);
	m_cPickList.SetFocus();
	focusSet = true;

	/*
	 * Tag area
	 */
	if (showTag)
		{
		m_cTagArea.EnableWindow(TRUE);
		m_cTagArea.ShowWindow(SW_SHOW);
		m_cTagArea.SetWindowText(strTag.c_str());
		}
	 else
		{
		m_cTagArea.EnableWindow(FALSE);
		m_cTagArea.ShowWindow(SW_HIDE);
		}

	/*
	 * User input box -- Not used in this style
	 */
	m_cUserInput.EnableWindow(FALSE);
	m_cUserInput.ShowWindow(SW_HIDE);

	/*
	 * Load the options list
	 */
	for (index=0;index<optionList.size();index++)
		{
		m_cPickList.AddString(optionList[index].c_str());
		}

	/*
	 * The message area overlaps the list. Since the option list and user area are at the same y coordinate, reduce the
	 * height of the message area by 1.5 times the height of the user area just like we did for MMI_Style_Edit1
	 */
	adjustWindow(m_cMessageArea,0,0,0,(int) (-1.5 * getWindowHeight(m_cUserInput)));
 
	/*
	 * The tag area and the buttons are covered up by the list. Also, the dialog itself is not tall enough. Make some
	 * adjustments.
	 *
	 * After some thinking about it, I realized that we just need to adjust everything by the difference in height between
     * the option list and the user area
	 */
	heightAdjustment = getWindowHeight(m_cPickList) - getWindowHeight(m_cUserInput);

	/*
	 * Resize the dialog window
	 */
	GetWindowRect(&windowRect);
	SetWindowPos(&wndTop,0,0,(windowRect.right-windowRect.left),(windowRect.bottom-windowRect.top+heightAdjustment),
		SWP_NOMOVE|SWP_NOACTIVATE);

	/*
	 * Scoot the controls down
	 */
	adjustWindow(m_cButton1,0,heightAdjustment,0,0);
	adjustWindow(m_cButton2,0,heightAdjustment,0,0);
	adjustWindow(m_cTagArea,0,heightAdjustment,0,0);

	break;
	}
 }

/*
 * We may need to manually fold blobs of text that are wider than the message area
 */
foldMessageText(strMessage);
fixNewLines(strMessage);

/*
 * See if the message area needs to be resized and make whatever adjustments are necessary
 */
m_cMessageArea.SetWindowText(strMessage.c_str());
adjustForMessageArea();

/*
 * The expected default is that the dialog is System Modal and top most.
 */
if (!disableSysModal)
	{
	/*
	 * Make the dialog System Modal and top most.
	 */
	ModifyStyle(0,DS_SYSMODAL);
	SetWindowPos(&wndTopMost,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_SHOWWINDOW);
	}
else
	{
	/*
	 * Just make the dialog start out on top
	 */
	SetWindowPos(&wndTop,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_SHOWWINDOW);
	}

/*
 * Center ourself relative to the PTS main window
 */
CenterWindow(FromHandle(findPTS()));
/* @Multiple instances */
if (m_strBdAddr != "")
{
	CString cs_caption;
	GetWindowText(cs_caption);
	cs_caption += " - ";
	cs_caption += m_strBdAddr.c_str();
	SetWindowText(cs_caption);
}
/*
 * Log the tag and message text
 */
if (fopen_s(&pLogFile,"implicit_send_log.txt","a") == 0)
	{
	fprintf_s(pLogFile,"--------------------------------------------------\n");
	fprintf_s(pLogFile,"%s\n",strTag.c_str());
	fprintf_s(pLogFile,"%s\n",strMessage.c_str());
	fprintf_s(pLogFile,"\n");
	fclose(pLogFile);
	}

return(focusSet?FALSE:TRUE);
}


/****************************************************************************************************************************
 **
 ** MainDialog::OnOK()
 **
 ** From section 5.1 of "Automating - Using Implicit Send":
 **
 **     "At various points during the execution of a test, the test case implementation can detect that an action requested
 **      via Implicit Send has occurred. When this happens, the test case may attempt to complete the ImplicitSendStyle()
 **      or ImplicitSendPinCode() operation that is in progress.
 **
 **      The mechanism used to do this is to send simulated button presses to a dialog whose title is User Action
 **      Required. ..."
 **
 ** What is actually sent from the test case is a WM_LBUTTONDOWN to the button whose ID is either IDOK or IDCANCEL depending
 ** on a parameter that is passed to the function to close the message box. The WM_LBUTTONDOWN is followed by a WM_LBUTTONUP
 ** to the same button.
 **
 ** We have hidden buttons on the dialog with IDs of IDOK and IDCANCEL to support this functionality. This is the handler for
 ** the IDOK button
 **/

void ImplicitSendDialog::OnOK()
{
	 
/*
 * Terminate the dialog
 */
m_strResponse.clear();
/*
 * EndDialog must be called within ui-thread, otherwise it would not be synchronized with RunModalLoop
 * EndDialog(DialogOkTerminationRequestedByTestCase);
 */
PostMessage(WM_IMPLICITSENDDIALOG, DialogOkTerminationRequestedByTestCase);
}

/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::adjustForMessageArea()
 **
 ** See if the message area needs to be resized and make whatever adjustments are necessary
 **/

void ImplicitSendDialog::adjustForMessageArea()
{
	int		deltaHeightAboveTag;
	int		deltaHeightBelowTag;
	int		deltaWidth;
	RECT	windowRect;

/*
 * Compute the best size for the message area as a delta from its current size
 */
m_cMessageArea.ComputeGoodSize(deltaWidth,deltaHeightAboveTag,1,1,true);
if (messageBoxFits(deltaWidth, deltaHeightAboveTag) == false)
	{
	/*
	 * If message does not fit into max size box, add scroll
	 */
	m_cMessageArea.ModifyStyle(0, WS_VSCROLL, 0);
	}

/*
 * If the display of the message tag is not enabled, remove the vertical space we reserved for it
 */
if (!m_cTagArea.IsWindowEnabled())
	{
	/*
	 * Take out the space for the tag and the blank space before and after it
	 */
	deltaHeightBelowTag = deltaHeightAboveTag - (3 * getWindowHeight(m_cTagArea));
	}
 else
	{
	/*
	 * No adjustment needed
	 */
	deltaHeightBelowTag = deltaHeightAboveTag;
	}

/*
 * Move controls that are active, note the ones that are above and below the tag area
 */
if (m_cButton1.IsWindowEnabled())
	{
	adjustWindow(m_cButton1,0,deltaHeightBelowTag,0,0);
	}

if (m_cButton2.IsWindowEnabled())
	{
	adjustWindow(m_cButton2,0,deltaHeightBelowTag,0,0);
	}

if (m_cMessageArea.IsWindowEnabled())
	{
	adjustWindow(m_cMessageArea,0,0,0,deltaHeightAboveTag);
	}

if (m_cOK.IsWindowEnabled())
	{
	adjustWindow(m_cOK,0,deltaHeightBelowTag,0,0);
	}

if (m_cPickList.IsWindowEnabled())
	{
	adjustWindow(m_cPickList,0,deltaHeightAboveTag,0,0);
	}

if (m_cTagArea.IsWindowEnabled())
	{
	adjustWindow(m_cTagArea,0,deltaHeightAboveTag,0,0);
	}

if (m_cUserInput.IsWindowEnabled())
	{
	adjustWindow(m_cUserInput,0,deltaHeightAboveTag,0,0);
	}

/*
 * Resize the dialog
 */
GetWindowRect(&windowRect);
SetWindowPos(&wndTop,0,0,(windowRect.right-windowRect.left),(windowRect.bottom-windowRect.top+deltaHeightBelowTag),
	SWP_NOMOVE|SWP_NOACTIVATE);
}



/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::adjustWindow()
 **
 ** Change the size and/or location of a control on the dialog
 **/

void ImplicitSendDialog::adjustWindow(
	CWnd&	window,
	int		deltaX,
	int		deltaY,
	int		deltaWidth,
	int		deltaHeight)
{
	RECT	dialogRect;
	int		height;
	int		width;
	RECT	windowRect;
	int		x;
	int		y;

/*
 * Get the window rectangle. These coordinates are relative to the upper left hand corner of display that MFC is creating
 * the dialog on
 */
window.GetWindowRect(&windowRect);

/*
 * Get the location of the dialog itself
 */
GetWindowRect(&dialogRect);

/*
 * Get the coordinates of the upper left hand corner adjusted to the display
 */
x = windowRect.left - dialogRect.left;
y = windowRect.top - dialogRect.top;

/*
 * Compute the height and width
 */
height = windowRect.bottom - windowRect.top;
width = windowRect.right - windowRect.left;

/*
 * Apply the changes
 */
x += deltaX;
y += deltaY;
height += deltaHeight;
width += deltaWidth;

/*
 * The x and y coordinates for MoveWindow() are relative to the client area of the dialog not the upper left hand
 * corner the GetWindowRect() gave us. Correct for that.
 */
x -= GetSystemMetrics(SM_CXDLGFRAME);										// The width of the window frame
y -= (GetSystemMetrics(SM_CYDLGFRAME) + GetSystemMetrics(SM_CYCAPTION));	// The height of the window frame and title bar

/*
 * Resize/relocate the window
 */
window.MoveWindow(x,y,width,height,FALSE);
}


/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::enumWindowsProc()
 **
 ** Callback function used by Win32's EnumWindows() function for inspecting each top level window
 **/

BOOL CALLBACK ImplicitSendDialog::enumWindowsProc(HWND hWnd, LPARAM lParam)
{
	BOOL	keepGoing = FALSE;

	if (::IsWindowVisible(hWnd))
	{
		/* Multiple instances */
		std::string s_title = "PTS - " + m_strBdAddr;
		char szWindowTitle[48];	// no more than 6 + 12 of s_title text and truncated the rest in the GetWindowText
		memset(szWindowTitle, 0, 48);
		::GetWindowText(hWnd, szWindowTitle, s_title.size() + 1);	// get truncated the size of string

		/*
		 * Does it start with the prefix above?
		 */
		if (strcmp(szWindowTitle, s_title.c_str()) == 0)
		{
			*(HWND*)lParam = hWnd;
			keepGoing = FALSE;
			SetLastError(ERROR_SUCCESS);
		}
		else
		{
			/*
			 * Not the window we are looking for
			 */
			keepGoing = TRUE;
		}
	}
	else
	{
		/*
		* Only care about visible windows. Important for Win 10
		*/
		keepGoing = TRUE;
	}

	return(keepGoing);
}


/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::findPTS()
 **
 ** Attempt to locate the top level window for PTS. If it is not found, return the handle for the desktop
 **/

HWND ImplicitSendDialog::findPTS()
{
	HWND	hWnd = NULL;

/*
 * Check an assumption
 */
C_ASSERT(sizeof(LPARAM) == sizeof(HWND*));

/*
 * Scan the top level windows
 */
if ((!::EnumWindows(&ImplicitSendDialog::enumWindowsProc,(LPARAM) &hWnd))
 && (hWnd != NULL))
	{
	/*
	 * We found the PTS window
	 */
	}
 else
	{
	/*
	 * We did not find a PTS window, use the desktop window instead
	 */
	hWnd = ::GetDesktopWindow();
	}

return(hWnd);
}

/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::foldBlob()
 **
 ** Called from foldMessageText(), this is actual operation of folding a large blob of text
 **/

void ImplicitSendDialog::foldBlob(
	std::string&	strBlob)
{
	bool					done;
	int						messageAreaWidth;
	CDC*					pDC;
	std::string				strSubBlob;
	std::string				strNewBlob;
	std::string::size_type	subBlobLength;
	CSize					textExtent;
	int						textWidth;
	bool					tooThin = false;
	bool					tooWide = false;

/*
 * How wide is the message area?
 */
messageAreaWidth = getWindowWidth(m_cMessageArea);

/*
 * Process the entire blob
 */
while (!strBlob.empty())
	{
	/*
	* How wide is the text blob?
	*/
	pDC = m_cMessageArea.GetDC();
	textExtent = pDC->GetTextExtent(strBlob.c_str(), strBlob.length());
	textWidth = textExtent.cx;

	/*
	 * Do we need to do anything?
	 */
	if (textWidth < messageAreaWidth)
		{
		/*
		 * Nope, we are done
		 */
		strNewBlob += strBlob;
		strBlob.clear();
		}
	 else
		{
		/*
		 * Start by guessing the size of the next chunk by using the ratio between the blob width and message area width
		 */
		subBlobLength = (int) (((double) messageAreaWidth * (double) strBlob.length()) / ((double) textWidth));

		/*
		 * We want an even number of characters in the blob. (The case that caused us to add the manual folding was a long
		 * string of hex data preceded by a 0x.)
		 */
		if ((subBlobLength % 2) == 1)
			{
			subBlobLength--;
			}

		/*
		 * The problem with basing the first guess on the text length is that the characters are variable width. The chunk
		 * may be smaller than we want, or larger
		 */
		done = false;
		tooThin = false;
		tooWide = false;
		while (!done)
			{
			/*
			 * Grab the next chunk
			 */
			strSubBlob = strBlob.substr(0,subBlobLength);

			/*
			 * How wide is it?
			 */
			textExtent = pDC->GetTextExtent(strSubBlob.c_str(),strSubBlob.length());
			textWidth = textExtent.cx;

			/*
			 * Too big or too small?
			 */
			if (textWidth < messageAreaWidth)
				{
				/*
				 * Possibly too small
				 */
				if (tooWide)
					{
					/*
					 * We have been making the chunk smaller and now it fits, we are done
					 */
					strNewBlob += strSubBlob + std::string("\n");
					strBlob = strBlob.substr(subBlobLength);
					done = true;
					}
				 else if (subBlobLength < strBlob.length())
					{
					/*
					 * See what happens if we make it wider
					 */
					subBlobLength += ((strBlob.length()-subBlobLength) == 1) ? 1 : 2;
					tooThin = true;
					}
				 else if (subBlobLength == strBlob.length())
					{
					/*
					 * We have run out of text
					 */
					strNewBlob += strSubBlob + std::string("\n");
					strBlob = strBlob.substr(subBlobLength);
					done = true;
					}
				}
			 else
				{
				/*
				 * Too big, what should we do?
				 */
				if (tooThin)
					{
					/*
					 * We have been making the chunk bigger and now it is too wide, the previous size was good
					 */
					subBlobLength -= 2;
					strSubBlob = strBlob.substr(0,subBlobLength);
					strNewBlob += strSubBlob + std::string("\n");
					strBlob = strBlob.substr(subBlobLength);
					done = true;
					}
				 else
					{
					/*
					 * We are too wide, make it smaller
					 */
					subBlobLength -= 2;
					tooWide = true;
					}
				}
			}
		}
	}

strBlob = strNewBlob;
}


/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::foldMessageText()
 **
 ** Take a pass over the message text to see if the caller sent us a large blob of non-whitespace delimited text. The CStatic
 ** control used for the message area will do line wrapping at whitespace boundaries, but it won't fold text blobs that are
 ** wider than the control. So, we need to find these blobs and wrap them ourself.
 **/

void ImplicitSendDialog::foldMessageText(
	std::string&			strMessage)
{
	std::string::size_type	base = 0;
	bool					collectingText;
	std::string::size_type	index = 0;
	std::string				strBlob;
	std::string				strOutput;

/*
 * Bail out early if there is nothing in the message
 */
if (strMessage.length() == 0)
	{
	goto ABORT;
	}

/*
 * Look at the first character to initalize our state
 */
collectingText = !isspace(strMessage[0]);

/*
 * Process every character in the message
 */
while (index < strMessage.length())
	{
	/*
	 * Are we collecting text or whitespace?
	 */
	if (collectingText)
		{
		/*
		 * We are collecting text, is the current character text or whitespace?
		 */
		if (!isspace(strMessage[index]))
			{
			/*
			 * Text, move on to the next character
			 */
			index++;
			}
		 else
			{
			/*
			 * Whitespace, get the collected text for further procesing
			 */
			strBlob = strMessage.substr(base,(index-base));
			foldBlob(strBlob);

			/*
			 * Append the (possibly) modified text blob to the output
			 */
			strOutput += strBlob;

			/*
			 * Reset and switch to collecting whitespace
			 */
			base = index;
			collectingText = false;
			}
		}
	 else
		{
		/*
		 * We are collecting whitespace, is the current character text or whitespace?
		 */
		if (!isspace(strMessage[index]))
			{
			/*
			 * Text, append the collected whitespace to the output string and reset
			 */
			strOutput += strMessage.substr(base,(index-base));
			base = index;
			collectingText = true;
			}
		 else
			{
			/*
			 * Whitespace, move on to the next character
			 */
			index++;
			}
		}
	}

/*
 * Append whatever is left to the output string
 */
strBlob = strMessage.substr(base,(index-base));
if (collectingText)
	{
	/*
	 * Fold if needed
	 */
	foldBlob(strBlob);
	}
strOutput += strBlob;

/*
 * Return the (possibly) updated string
 */
strMessage = strOutput;

ABORT:
return;
}

/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::getOptionFlags()
 **
 ** Extract the option flags from PTS.ini
 **/

void ImplicitSendDialog::getOptionFlags(
	bool&	disableSysModal,
	bool&	showTag,
	bool&	showBorder)
{
	char	szPtsIniPath[MAX_PATH];

/*
 * PTS.ini is in the same folder as PTS.exe, which is the main module for the current process. This makes finding PTS.ini
 * pretty easy
 *
 * Get the path to AppData folder
 */
SHGetFolderPath(NULL, CSIDL_APPDATA | CSIDL_FLAG_CREATE, NULL, 0, szPtsIniPath);

/*
 * Morph to PTS.ini
 */
std::string sSIGAppDataPath = szPtsIniPath;
sSIGAppDataPath += "\\Bluetooth_SIG\\ProfileTuningSuite_6";

sprintf_s(szPtsIniPath, _countof(szPtsIniPath), "%s\\pts.ini", sSIGAppDataPath.c_str());

/*
 * Get the option flags
 */
disableSysModal = (GetPrivateProfileInt("ImplicitSend","disableSysModal",0,szPtsIniPath) != 0);
showTag = (GetPrivateProfileInt("ImplicitSend","showTag",0,szPtsIniPath) != 0);
showBorder = (GetPrivateProfileInt("ImplicitSend","showBorder",0,szPtsIniPath) != 0);
}


/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::getWindowHeight()
 **
 ** Get the height of a control on the dialog
 **/

int ImplicitSendDialog::getWindowHeight(
	CWnd&	window)
{
	RECT	windowRect;

/*
 * Get the window rectangle.
 */
window.GetWindowRect(&windowRect);

/*
 * Compute and return the height
 */
return(windowRect.bottom - windowRect.top);
}


/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::getWindowWidth()
 **
 ** Get the width of a control on the dialog
 **/

int ImplicitSendDialog::getWindowWidth(
	CWnd&	window)
{
	RECT	windowRect;

/*
 * Get the window rectangle.
 */
window.GetWindowRect(&windowRect);

/*
 * Compute and return the width
 */
return(windowRect.right - windowRect.left);
}


/****************************************************************************************************************************
 **
 ** ImplicitSendDialog::processMmiText()
 **
 ** Extract the various bits of info from the MMI string passed to ImplicitSendStyle()
 **/

void ImplicitSendDialog::processMmiText(
	const std::string&			strMmiText,
	UINT						mmiStyle,
	std::string&				strMessage,
	std::string&				strTag,
	std::vector<std::string>&	optionList)
{
	bool						gotMessage = false;
	std::string::size_type		optionEnd;
	std::string					strTemp;
	std::string					strText;
	std::string::size_type		tagEnd;
	std::string::size_type		tagStart;

/*
 * Copy the string so we can modify it
 */
strText = strMmiText;
std::string::size_type nextPos = strText.find("&");

// Replace all single instances of '&' with '&&', we don't want underlines in the dialog box
while (nextPos != std::string::npos)
{
	strText.replace(nextPos, 1, "&&");
	nextPos = strText.find("&", nextPos + 2);
}

/*
 * Is there a tag present? The tag is delimited by curly braces "{}".
 */
tagStart = strText.find('{');
tagEnd = strText.find('}',tagStart+1);

if ((tagStart != std::string::npos)
 && (tagEnd != std::string::npos))
	{
	/*
	 * A tag is present, extract it and remove it from the working copy of the text
	 */
	strTag = strText.substr(tagStart,tagEnd-tagStart+1);
	strText.erase(tagStart,tagEnd-tagStart+1);
	}
 else
	{
	/*
	 * No tag present
	 */
	strTag = "{<No Tag Present>}";
	}

/*
 * Depending on the style, the remaining text is all of the message or the message plus selection options
 */
if (mmiStyle != MMI_Style_Edit2)
	{
	/*
	 * The rest of the text is the message and there are no selection options
	 */
	strMessage = strText;
	optionList.clear();
	}
 else
	{
	/*
	 * The text contains the prompt to be displayed to the user followed by options for the selection list. The message
	 * and each option are terminated by newlines. The end of the list is a zero length option string.
	 */
	while (!strText.empty())
		{
		/*
		 * Find the next newline
		 */
		optionEnd = strText.find('\n');

		/*
		 * Extract the string, and remove it from the working copy of the text
		 */
		if (optionEnd != std::string::npos)
			{
			/*
			 * Extract and remove
			 */
			strTemp = strText.substr(0,optionEnd);
			strText.erase(0,optionEnd+1);
			}
		 else
			{
			/*
			 * No new line (even thought there should be). Just use the rest of the string and clear it
			 */
			strTemp = strText;
			strText.clear();
			}

		/*
		 * What should we do with the text we just extracted?
		 */
		if (strTemp.empty())
			{
			/*
			 * Nothing, we are done
			 */
			}
		 else if (!gotMessage)
			{
			/*
			 * This is the first piece, return it as the message
			 */
			strMessage = strTemp;
			gotMessage = true;
			}
		 else
			{
			/*
			 * This is an option, add it to the list
			 */
			optionList.push_back(strTemp);
			}
		}
	}

/*
 * Finally, do a little formatting on the tag. We'll leave off the "MMI_Style_" part to help the string fit in the space
 * alloted on the dialog
 */
switch (mmiStyle)
 {
 case MMI_Style_Ok_Cancel1: 	strTag = "Style: Ok_Cancel1, Tag: " + strTag;		break;
 case MMI_Style_Ok_Cancel2: 	strTag = "Style: Ok_Cancel2, Tag: " + strTag;		break;
 case MMI_Style_Ok1:			strTag = "Style: Ok1, Tag: " + strTag;				break;
 case MMI_Style_Yes_No1:		strTag = "Style: Yes_No1, Tag: " + strTag;			break;
 case MMI_Style_Yes_No_Cancel1: strTag = "Style: Yes_No_Cancel1, Tag: " + strTag;	break;
 case MMI_Style_Abort_Retry1:	strTag = "Style: Abort_Retry1, Tag: " + strTag;		break;
 case MMI_Style_Edit1:			strTag = "Style: Edit1, Tag: " + strTag;			break;
 case MMI_Style_Edit2:			strTag = "Style: Edit2, Tag: " + strTag;			break;
 }
}

void ImplicitSendDialog::getMaxMsgWindowSize(int& maxWidth, int& maxHeight)
{
	maxWidth = MAX_WIDTH;
	maxHeight = MAX_HEIGHT;

	RECT rect;
	m_cMessageArea.GetWindowRect(&rect);
	m_cMessageArea.GetParent()->ScreenToClient(&rect);
	maxWidth -= rect.left;
	maxHeight -= rect.top;

	CWnd* pBellowWnd = NULL;
	if (m_cPickList.IsWindowEnabled())
	{
		pBellowWnd = static_cast<CWnd*>(&m_cPickList);
	}
	else if (m_cUserInput.IsWindowEnabled())
	{
		pBellowWnd = static_cast<CWnd*>(&m_cUserInput);
	}
	else if (m_cTagArea.IsWindowEnabled())
	{
		pBellowWnd = static_cast<CWnd*>(&m_cTagArea);
	}
	else
	{
		pBellowWnd = static_cast<CWnd*>(&m_cButton1);
	}

	if (pBellowWnd != NULL)
	{
		pBellowWnd->GetWindowRect(&rect);
		pBellowWnd->GetParent()->ScreenToClient(&rect);
		maxHeight -= rect.top;
	}	
}

bool ImplicitSendDialog::messageBoxFits(int& recWidthDelta, int& recHeightDelta)
{
	bool retval = true;
	int maxWidth = 0;
	int maxHeight = 0;
	getMaxMsgWindowSize(maxWidth, maxHeight);
	int currentHeight = getWindowHeight(m_cMessageArea);
	int currentWidth = getWindowWidth(m_cMessageArea);
	if (currentHeight + recHeightDelta > maxHeight)
	{
		recHeightDelta = maxHeight - currentHeight;
		retval = false;
	}
	if (currentWidth + recWidthDelta > maxWidth)
	{
		recWidthDelta = maxWidth - currentWidth;
		retval = false;
	}
	return retval;
}

void ImplicitSendDialog::fixNewLines(std::string& strMessage)
{
	size_t i = strMessage.find('\n', 0);
	while (i != std::string::npos && i < strMessage.length())
	{
		if (i == 0 || strMessage[i - 1] != '\r')
		{
			strMessage.replace(i, 1, "\r\n");
			i ++;	// increment index to account for size increase
		}
		i = strMessage.find('\n', i + 1); // perform find from next index and not work with the same \n
	}

}
