/***
 *** ImplicitSendDialog.h
 ***
 *** This is the source code for the standard implicit_send3.dll that is provided with the Profile Tuning Suite (PTS). It
 *** is provided to PTS users as a starting point for developing their own Implict Send DLL.
 ***
 *** Please see the "Automating - Using Implicit Send" document for more information. This document can be found in the
 *** same folder as this source code, or by selecting "Reference Documents" -> "Automating" from the PTS Start page.
 ***
 *** ----------
 ***
 *** This module provides the dialog box that is used to interact with the test operator.
 ***
 *** ----------
 ***
 *** Copyright 2012, Bluetooth SIG, Inc.
 ***
 *** ----------
 ***
 *** Tom Allebrandi (tallebrandi@bluetooth.com)
 *** Bluetooth SIG, Inc.
 ***/

#if	(!defined(IMPLICITSENDDIALOG_H_INCLUDED))
#define	IMPLICITSENDDIALOG_H_INCLUDED

/****************************************************************************************************************************
 **
 ** Includes (needed by the class)
 **/
#include	<string>
#include	<vector>
#include	"mycstatic.h"

/****************************************************************************************************************************
 **
 ** ImplicitSendDialog Class
 **/

#define WM_IMPLICITSENDDIALOG (WM_USER+7)

class ImplicitSendDialog : public CDialog
{
/*
 * Exit codes from the dialog
 */
public:
enum	{
		AffirmativeResponse,
		NegativeResponse,
		DialogOkTerminationRequestedByTestCase,
		DialogCancelTerminationRequestedByTestCase
		};
		
/*
 * Construction/Destruction
 */
public:
				ImplicitSendDialog(void);
				~ImplicitSendDialog(void);

/*
 * Interface
 */
public:
	std::string	GetData(void);	// Return data from the dialog for the MMI_Style_Edit1 and MMI_Style_Edit2 styles
	void		SetData(const std::string& strMmiText,UINT mmiStyle,const std::string& strBdAddr);	// Provide the data for the dialog


/*
 * Class internal functions
 */
private:
	/*
	 * Window message handlers
	 */
	void		DoDataExchange(CDataExchange* pDx);
	void		OnButton1(void);
	void		OnButton2(void);
	void		OnCancel(void);
	BOOL		OnInitDialog(void);
	void		OnOK(void);
	afx_msg LRESULT OnMessageReceived(WPARAM wParam, LPARAM lParam);

	DECLARE_MESSAGE_MAP();

	/*
	 * Utilities and helpers
	 */
	void		adjustForMessageArea(void);
	void		adjustWindow(CWnd& window,int x,int y,int width,int height);
	HWND		findPTS(void);
	void		foldBlob(std::string& strBlob);
	void		foldMessageText(std::string& strMessage);
	void		getOptionFlags(bool& disableSysModal,bool& showTag,bool& showBorder);
	int			getWindowHeight(CWnd& window);
	int			getWindowWidth(CWnd& window);
	void		processMmiText(const std::string& strMmiText,UINT mmiStyle,std::string& strMessage,std::string& strTag,
					std::vector<std::string>& optionList);
	void		getMaxMsgWindowSize(int& maxWidth, int& maxHeight);
	bool		messageBoxFits(int&	recWidthDelta, int&	recHeightDelta);
	void		fixNewLines(std::string& strMessage);

static BOOL CALLBACK enumWindowsProc(HWND hWnd,LPARAM lParam);

/*
 * Class data
 */
private:
	/*
	 * Controls
	 */
	CButton		m_cButton1;		// Left hand button on the dialog
	CButton		m_cButton2;		// Right hand button on the dialog
	MyCEdit		m_cMessageArea;	// Message display area
	CButton		m_cOK;			// A hidden button, see the OnOK() method for the reason why we need it
	CListBox	m_cPickList;	// List of options for MMI_Style_Edit2
	CStatic		m_cTagArea;		// Place that displays the message tag if that option is enabled
	CEdit		m_cUserInput;	// Data entry control for MMI_Style_Edit1

	/*
	 * Other data
	 */
	UINT		m_mmiStyle;		// MMI style value supplied to ImplicitSendStyle()
	std::string	m_strMmiText;	// Message text supplied to ImplicitSendStyle()
	std::string	m_strResponse;	// User input for MMI_Style_Edit1, chosen option for MMI_Style_Edit2
};
#endif
