#ifndef __MSXMLWRAPPER_H__
#define __MSXMLWRAPPER_H__

#import <msxml6.dll> exclude("IStream", "ISequentialStream", "_LARGE_INTEGER", "_ULARGE_INTEGER", "tagSTATSTG", "_FILETIME")

class CMSXMLWrapper {

public : 

	CMSXMLWrapper( void )
	{
		HRESULT hr = CoInitialize( NULL );
		if ( (hr != S_OK) && (hr != S_FALSE) )
		{
			throw std::exception( "CMSXMLWrapper::CMSXMLWrapper, CoInitialize Failed!" );
		}
		else
		{
			// Do nothing
		}
	}
	virtual ~CMSXMLWrapper( void )
	{
		CoUninitialize();
	}

	MSXML2::IXMLDOMDocumentPtr LoadFromString( const std::string &xmlString )
	{
		// Create the document
		MSXML2::IXMLDOMDocumentPtr xmlDocument;
		HRESULT hr = CoCreateInstance( __uuidof(MSXML2::DOMDocument60), NULL, CLSCTX_INPROC_SERVER, IID_IXMLDOMDocument, (void**)&xmlDocument );
		if ( hr == S_OK )
		{
			// set document flags
			xmlDocument->async = VARIANT_FALSE;

			// load from file name
			VARIANT_BOOL status = xmlDocument->loadXML( xmlString.c_str() );

			// check status
			if ( status == VARIANT_TRUE )
			{
				if ( hr == S_OK )
				{
					return xmlDocument;
				}
			}
			
			// failed to load document do nothing
			xmlDocument.Release();
		}
		else
		{
			// do nothing
		}
		
		// failed
		return NULL;
	}

	MSXML2::IXMLDOMNodePtr LoadFromFile( const std::string &xmlFileName )
	{
		FILE *pFile = NULL;
		if ( fopen_s( &pFile, xmlFileName.c_str(), "rb" ) == 0 )
		{
			fseek( pFile, 0, SEEK_END );
			long fileSize = ftell( pFile );
			rewind( pFile );
			char *pBuffer = new char [fileSize+1];
			fread( pBuffer, 1, fileSize, pFile );
			fclose( pFile );
			pBuffer[fileSize] = 0;
			MSXML2::IXMLDOMNodePtr result = LoadFromString( pBuffer );
			delete [] pBuffer;
			return result;
		}
		else
		{
			// do nothing
		}

		return NULL;
	}

	MSXML2::IXMLDOMNodePtr LoadFromFileAsString( const std::string &xmlFileName )
	{
		// Create the document
		MSXML2::IXMLDOMDocumentPtr xmlDocument;
		HRESULT hr = CoCreateInstance( __uuidof(MSXML2::DOMDocument60), NULL, CLSCTX_INPROC_SERVER, IID_IXMLDOMDocument, (void**)&xmlDocument );
		if ( hr == S_OK )
		{
			// load from file name
			VARIANT_BOOL status = xmlDocument->load( xmlFileName.c_str() );

				// check status
			if ( status == VARIANT_TRUE )
			{
				if ( hr == S_OK )
				{
					return xmlDocument;
				}
			}
			
			// failed to load document do nothing
			xmlDocument.Release();
		}
		else
		{
			// do nothing
		}
		
		// failed
		return NULL;
	}
};

#endif // __MSXMLWRAPPER_H__